/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  Mine.cpp
 *  Racer
 *
 *  Created by Nathan Sturtevant on Thu Apr 08 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include "point3d.h"
#include "vector3d.h"
#include "Mine.h"
#include "Fragment.h"

Mine::Mine(GLfloat *params, double _time) // start coord (3), velocity(3)
:Object(kMineObject, kShipObject)
{
	for (int x = 0; x < 6; x++)
		param[x] = params[x];
	elapsedTime = 0;
	state = kInert;
	time = _time;
	//deathSound = 0;
}

Mine::Mine(GLfloat posx, GLfloat posy, GLfloat posz, GLfloat velx, GLfloat vely, GLfloat velz, double _time) // start coord (3), velocity(3)
:Object(kMineObject, kShipObject)
{
	param[0] = posx;
	param[1] = posy;
	param[2] = posz;
	param[3] = velx;
	param[4] = vely;
	param[5] = velz;
	elapsedTime = 0;
	state = kInert;
	time = _time;
	//deathSound = 0;
}

void Mine::draw()
{
	if (state == kInert)
	{
		glColor3f(0.0, 1-elapsedTime/2, 1);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		drawSphere(0.05, 8*(gDetail?4:1));
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	}
	else if (state == kArmed)
	{
		glColor3f(0, 0, 1.0);
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		drawSphere(0.05+.01*sin(2*elapsedTime-4), 8*(gDetail?2:1));
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glColor3f(0, 0, 0);
		drawSphere(0.05+.01*sin(2*elapsedTime-4), 8*(gDetail?2:1));
	}
}

uint32_t Mine::stepTime(double amount)
{
	elapsedTime += amount;
	//for (int x = 0; x < 3; x++)
	//	param[x]+=amount*param[x+3];
	if ((elapsedTime > 2) && (nextGoal == 0))
		state = kArmed;
	if ((state == kArmed) && (elapsedTime > time) && (!activeGoal))
	{
		destroy(); // to get sound effect
		return kObjectDestroyed;//|kNewObjectsCreated;
	}
	return kObjectRemains;
}

void Mine::drawSphere(double tRadius, int precision)
{
  int i,j;
	int n = precision;
	double r = tRadius;
  double theta1,theta2,theta3;
  point3d e,p,c(0, 0, 0);
  
 	glTranslatef(param[0], param[1], param[2]);
	if (r < 0)
		r = -r;
	if (n < 0)
		n = -n;
	if (n < 4 || r <= 0) {
		glBegin(GL_POINTS);
		glVertex3f(c.x,c.y,c.z);
		glEnd();
	}
	else {
		for (j=0/*n/4*/;j<n/2;j++) {
			theta1 = j * TWOPI / n - PID2;
			theta2 = (j + 1) * TWOPI / n - PID2;
			
			glBegin(GL_QUAD_STRIP);
			//glBegin(GL_TRIANGLE_STRIP);
			//glBegin(GL_LINE_STRIP);
			for (i=0;i<=n;i++) {
				theta3 = i * TWOPI / n;
				
				e.x = cos(theta2) * cos(theta3);
				e.y = sin(theta2);
				e.z = cos(theta2) * sin(theta3);
				p.x = c.x + r * e.x;
				p.y = c.y + r * e.y;
				p.z = c.z + r * e.z;
				
				glNormal3f(-e.x,-e.y,-e.z);
				//glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
				glVertex3f(p.x,p.y,p.z);
				
				e.x = cos(theta1) * cos(theta3);
				e.y = sin(theta1);
				e.z = cos(theta1) * sin(theta3);
				p.x = c.x + r * e.x;
				p.y = c.y + r * e.y;
				p.z = c.z + r * e.z;
				
				glNormal3f(-e.x,-e.y,-e.z);
				//glTexCoord2f(i/(double)n,2*j/(double)n);
				glVertex3f(p.x,p.y,p.z);
			}
			glEnd();
		}
	}
 	glTranslatef(-param[0], -param[1], -param[2]);
}

//Object *Mine::getNewObject()
//{
////	if (fragCount > 0)
////	{
////		fragCount--;
////		// location, speed, rotational velocity
////	}
//	return 0;
//}

point3d Mine::getLocation()
{
	point3d p;
	p.x = param[0];
	p.y = param[1];
	p.z = param[2];
	return p;
}

double Mine::getRadius()
{
	return 0.06;
}

uint32_t Mine::testCollision(Object *o)
{
	if (state == kInert) // we don't destroy anything when inert
		return kNothingDestroyed;
	vector3d p1 = getLocation().getVector(o->getLocation());
	float rad = getRadius()+o->getRadius();
	if (p1.squaredLength() < rad*rad)
		return kBothDestroyed;
	return kNothingDestroyed;
}	

void Mine::destroy()
{
  if ((nextGoal) && (activeGoal))
  {
    activeGoal = false;
    nextGoal->activate();
    nextGoal = 0;
  }
  if (nextGoal == 0)
  {
    int notes[4] = {0, 1, 70, 120};
    pushCreatedObject(new Music(notes, 4, kPlayOnce, 0, 128*4));
    
    for (int x = 0; x < 8; x++)
      pushCreatedObject(new Fragment(param[0], param[1], param[2],
                                     rand01()*.1-.05, rand01()*.1-.05, rand01()*.1-.05,
                                     rand01()*360, rand01()*360));
    elapsedTime += time;
  }
}

void Mine::setColor()
{
	if (state == kInert)
		glColor3f(0, 0, .5);
	else
		glColor3f(0, 0, 1);
}

void Mine::activate()
{
  state = kArmed;
  activeGoal = true;
}
