/*
 (MIT License)
 
 Copyright (c) 2004 NS Software
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to the following
 conditions:
 
 The above copyright notice and this permission notice shall be included in all copies
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
/*
 *  World.cpp
 *  Carbon OpenGL
 *
 *  Created by Nathan Sturtevant on Wed Mar 24 2004.
 *  Copyright (c) 2004 NS Software.
 *
 */

#include "World.h"

World::World()
{
}

World::~World()
{
	for (unsigned int x = 0; x < otherObjs.size(); x++)
	{
    delete otherObjs[x];
		otherObjs[x] = 0;
	}
	for (unsigned int x = 0; x < displayObjs.size(); x++)
	{
    delete displayObjs[x];
		displayObjs[x] = 0;
	}
	for (unsigned int x = 0; x < disabledObjs.size(); x++)
	{
    delete disabledObjs[x];
		disabledObjs[x] = 0;
	}
}

void World::reset()
{
	for (unsigned int x = 0; x < otherObjs.size(); x++)
	{
    delete otherObjs[x];
		otherObjs[x] = 0;
	}
	while (otherObjs.size() > 0)
		otherObjs.pop_back();
	for (unsigned int x = 0; x < disabledObjs.size(); x++)
	{
    delete disabledObjs[x];
		disabledObjs[x] = 0;
	}
	while (disabledObjs.size() > 0)
		disabledObjs.pop_back();
	for (unsigned int x = 0; x < displayObjs.size(); x++)
	{
    delete displayObjs[x];
		displayObjs[x] = 0;
	}
	while (displayObjs.size() > 0)
		displayObjs.pop_back();
}

void World::addObject(Object *obj)
{
	if (obj->getType() == kDisplayObject)
		displayObjs.push_back(obj);
	else
		otherObjs.push_back(obj);
}

void World::stepTime(double amount)
{
	int result;

	unsigned int oSize = otherObjs.size();
	for (unsigned int x = 0; x < oSize; x++)
	{
    Object *xx = otherObjs[x];
    result = xx->stepTime(amount);
		for (Object *obj = xx->getNewObject(); obj; obj = xx->getNewObject())
		{
			if (obj)
			{
				addObject(obj);
				//oSize++;
			}
		}
		if (result&kObjectDestroyed) // remove from simulation
		{
			delete xx;
			otherObjs[x] = otherObjs[otherObjs.size()-1];
			otherObjs.pop_back();
			x--; oSize--;
		}
		if (result&kObjectDisabled)  // remove from simulation, but don't delete
		{
				disabledObjs.push_back(xx);
				otherObjs[x] = otherObjs[otherObjs.size()-1];
				otherObjs.pop_back();
				x--; oSize--;
		}
	}
	// draw all display after other objects...
	oSize = displayObjs.size();
	for (unsigned int x = 0; x < oSize; x++)
	{
    Object *xx = displayObjs[x];
    result = xx->stepTime(amount);
		for (Object *obj = xx->getNewObject(); obj; obj = xx->getNewObject())
		{
			if (obj)
			{
				addObject(obj);
				//oSize++;
			}
		}
		if (result&kObjectDestroyed) // remove from simulation
		{
			delete xx;
			displayObjs[x] = displayObjs[displayObjs.size()-1];
			displayObjs.pop_back();
			x--; oSize--;
		}
		if (result&kObjectDisabled)  // remove from simulation, but don't delete
		{
			disabledObjs.push_back(xx);
			displayObjs[x] = displayObjs[displayObjs.size()-1];
			displayObjs.pop_back();
			x--; oSize--;
		}
	}

	// now check for collisions
	oSize = otherObjs.size();
	for (unsigned int x = 0; x < oSize; x++)
	{
    Object *xx = otherObjs[x];
		if (xx->getType() == kShipObject)
			continue; // ship never hits anything, so avoid testing
		for (unsigned int y = 0; y < oSize; y++)
		{
      Object *yy = otherObjs[y];
			if (x == y)
				continue;
      if (xx->testAgainst(yy->getType()))
      {
        int result = xx->testCollision(yy);
        switch (result) {
          case kNothingDestroyed:
            break;
          case kThisDestroyed:
            xx->destroy();
            break;
          case kThatDestroyed:
            yy->destroy();
            break;
          case kBothDestroyed:
            xx->destroy();
            yy->destroy();
            break;
        }
			}
		}
	}
}

void World::draw()
{
	unsigned int oSize = otherObjs.size()-1;
	for (int x = oSize; x >= 0; x--)
	{
    otherObjs[x]->draw();
	}
	oSize = displayObjs.size()-1;
	for (int x = oSize; x >= 0; x--)
	{
    displayObjs[x]->draw();
	}
}

void World::drawDisplayObjs() // for when paused
{
	unsigned int oSize = otherObjs.size()-1;
	for (int x = oSize; x >= 0; x--)
	{
    displayObjs[x]->draw();
	}
}

void World::drawOverview() // don't show display objects
{
	Object *ship=0, *target=0;
	glPointSize(3);
	glBegin(GL_POINTS);
	//glColor3f(1, 1, 1);
	unsigned int oSize = otherObjs.size()-1;
	for (int x = oSize; x >= 0; x--)
	{
    Object *xx = otherObjs[x];
		point3d where = xx->getLocation();
		if (xx->getType() == kShipObject)
			ship = xx;
		if (xx->isGoal())
			target = xx;
		xx->setColor();
		glVertex3f(where.x, where.y, where.z);
	}
	glEnd();
	glPointSize(2);
	if (ship && target)
	{
		point3d where;
		glBegin(GL_LINES);
		glColor3f(.7, .7, .7);
		where = ship->getLocation();
		glVertex3f(where.x, where.y, where.z);
		where = target->getLocation();
		glVertex3f(where.x, where.y, where.z);
		glEnd();
	}
	glPointSize(1);
}
